/*** Z80Em: Portable Z80 emulator *******************************************/
/***                                                                      ***/
/***                              Z80Codes.h                              ***/
/***                                                                      ***/
/*** This file contains various macros used by the emulation engine       ***/
/***                                                                      ***/
/*** Copyright (C) Marcel de Kogel 1996,1997                              ***/
/***     You are not allowed to distribute this software commercially     ***/
/***     Please, notify me, if you make any changes to this file          ***/
/****************************************************************************/

void M_POP(word *Rg);
void M_PUSH(word Rg);
void M_CALL(void);
void M_JP(void);
void M_JR(void);
void M_RET(void);
void M_RST(word Addr);
void M_BIT(int Bit,byte Reg);
void M_AND(byte Reg);
void M_OR(byte Reg);
void M_XOR(byte Reg);

#define	M_IN(Reg)	Reg=DoIn_N;									\
					R.AF.B.l = (byte)((R.AF.B.l & C_FLAG) | ZSPTable[Reg])

void M_RLCA(void);
void M_RRCA(void);
void M_RLA(void);
void M_RRA(void);
void M_ADD(byte Reg);
void M_ADC(byte Reg);
void M_SUB(byte Reg);
void M_SBC(byte Reg);
void M_CP(byte Reg);


#define M_SKIP_CALL     R.PC.W.l+=2
#define M_SKIP_JP       R.PC.W.l+=2
#define M_SKIP_JR       R.PC.W.l+=1
#define M_SKIP_RET

#define M_SET(Bit,Reg)  Reg|=1<<Bit
#define M_RES(Bit,Reg)  Reg&=~(1<<Bit)

#define M_RLC(Reg)         \
{                          \
 int q;                    \
 q=Reg>>7;                 \
 Reg=(Reg<<1)|q;           \
 R.AF.B.l=ZSPTable[Reg]|q; \
}
#define M_RRC(Reg)         \
{                          \
 int q;                    \
 q=Reg&1;                  \
 Reg=(Reg>>1)|(q<<7);      \
 R.AF.B.l=ZSPTable[Reg]|q; \
}
#define M_RL(Reg)            \
{                            \
 int q;                      \
 q=Reg>>7;                   \
 Reg=(Reg<<1)|(R.AF.B.l&1);  \
 R.AF.B.l=ZSPTable[Reg]|q;   \
}
#define M_RR(Reg)            \
{                            \
 int q;                      \
 q=Reg&1;                    \
 Reg=(Reg>>1)|(R.AF.B.l<<7); \
 R.AF.B.l=ZSPTable[Reg]|q;   \
}
#define M_SLL(Reg)           \
{                            \
 int q;                      \
 q=Reg>>7;                   \
 Reg=(Reg<<1)|1;             \
 R.AF.B.l=ZSPTable[Reg]|q;   \
}
#define M_SLA(Reg)           \
{                            \
 int q;                      \
 q=Reg>>7;                   \
 Reg<<=1;                    \
 R.AF.B.l=ZSPTable[Reg]|q;   \
}
#define M_SRL(Reg)           \
{                            \
 int q;                      \
 q=Reg&1;                    \
 Reg>>=1;                    \
 R.AF.B.l=ZSPTable[Reg]|q;   \
}
#define M_SRA(Reg)           \
{                            \
 int q;                      \
 q=Reg&1;                    \
 Reg=(Reg>>1)|(Reg&0x80);    \
 R.AF.B.l=ZSPTable[Reg]|q;   \
}

#define M_INC(Reg)										\
	++Reg;												\
	R.AF.B.l = (byte)((R.AF.B.l&C_FLAG)|ZSTable[Reg]|	\
			((Reg==0x80)?V_FLAG:0)|((Reg&0x0F)?0:H_FLAG))

#define M_DEC(Reg)										\
	R.AF.B.l = (byte)((R.AF.B.l&C_FLAG)|N_FLAG|			\
		((Reg==0x80)?V_FLAG:0)|((Reg&0x0F)?0:H_FLAG));	\
	R.AF.B.l |= ZSTable[--Reg]


#define M_ADDW(Reg1,Reg2)								\
{														\
	dword	q;											\
	q = (dword)R.Reg1.D + (dword)R.Reg2.D;				\
	R.AF.B.l=(byte)((R.AF.B.l&(S_FLAG|Z_FLAG|V_FLAG))|	\
			(((R.Reg1.D^q^R.Reg2.D)&0x1000)>>8)|		\
			((q>>16)&1));								\
	R.Reg1.W.l=(word)q;									\
}

#define M_ADCW(Reg)												\
{																\
	dword	q;													\
	q = (dword)R.HL.D + (dword)R.Reg.D + (dword)(R.AF.D&1);		\
	R.AF.B.l = (byte)((((R.HL.D^q^R.Reg.D)&0x1000l)>>8)|		\
			((q>>16)&1)|										\
			((q&0x8000l)>>8)|									\
			((q&65535)?0:Z_FLAG)|								\
			(((R.Reg.D^R.HL.D^0x8000)&(R.Reg.D^q)&0x8000)>>13));\
	R.HL.W.l = (word)q;											\
}

#define M_SBCW(Reg)										\
{														\
	dword	q;											\
	q = (dword)R.HL.D-(dword)R.Reg.D-(dword)(R.AF.D&1);	\
	R.AF.B.l=(byte)((((R.HL.D^q^R.Reg.D)&0x1000l)>>8)|	\
			((q>>16)&1)|((q&0x8000l)>>8)|				\
			((q&0x0000ffffl)?0:Z_FLAG)|					\
			(((R.Reg.D^R.HL.D)&(R.HL.D^q)&0x8000l)>>13)|	/* ASG 220997 */ \
			N_FLAG);									\
	R.HL.W.l = (word)q;									\
}


